import java.rmi.*;
import java.rmi.server.*;
import java.util.*;

public class CallbackHelloServerImpl extends UnicastRemoteObject
		implements CallbackHelloServerInterface {

	/* lista dei client registrati */ 
	private List<CallbackHelloClientInterface> clients;

	/* crea un nuovo servente */ 
	public CallbackHelloServerImpl() throws RemoteException {
		super();
		clients = new ArrayList<CallbackHelloClientInterface>();
	}

	/* metodo di business - quando viene richiamato, fa il callback a tutti i client registrati */ 
    public String sayHello(String name) throws RemoteException {
    	doCallbacks(); 
		return "Hello, " + name + "!";
	}

    /* registrazione per il callback */
	public void registerForCallback(CallbackHelloClientInterface callbackClient)
	  	throws RemoteException {
		if (!clients.contains(callbackClient)) {
			clients.add(callbackClient);
			System.out.println("### New client registered.");
		}
	}

	/* annulla registrazione per il callback */
	public synchronized void unregisterForCallback(
			CallbackHelloClientInterface callbackClient)
	   	throws RemoteException {
		if (clients.remove(callbackClient)) {
			System.out.println("### Client unregistered.");
		} else {
			System.out.println("### Unable to unregister client.");
		}
	}

	/* gestione dei callback - viene richiamato ogni volta che si registra un nuovo client */
	private synchronized void doCallbacks() throws RemoteException {
		System.out.println("### Starting callbacks.");
		Iterator i = clients.iterator();
		int numeroClienti = clients.size(); 
		int n = 1;
		while (i.hasNext()) {
			CallbackHelloClientInterface client = (CallbackHelloClientInterface) i.next();
			client.notifyMe("Sei il client #" + n + " di " + numeroClienti);
			n++;
		}
		System.out.println("### Callbacks complete.");
	}
}

